
#define NUM_TASKS 4
#define COEXEC_VERBOSE
#use co_exec.lib

#define STK_SIZE 2048

#define Stack NULL

TASK tasks[NUM_TASKS];

int Task1( int iParam );
int StackTask(int param);
int EventTask( int Param );

char gevent;
int fast_switch;
unsigned long switches;

root void swtime(void);

#asm root nodebug
swtime::
loop:
		ld		hl,(switches)
      inc	hl
      ld		(switches),hl
      jr		nz,.tswitch
		ld		hl,(switches+2)
      inc	hl
      ld		(switches+2),hl
.tswitch:
		lcall	task_switch
		ld		a,(fast_switch)
      or	a
      ret	z
      jr		loop
#endasm

root nodebug void test_speed(void)
{
   fast_switch=1;
	switches=0l;
   sleep(1000l);
   fast_switch=0;
	printf("%ld task switches/sec.\n",switches);
   // 44mhz is 1.2us per loop
   printf("\tApprox %f us per switch at 44mHz.\n",(1000000.0/switches)-1.156);
}

void main(void)
{
	int i;
   puts("Startup.\n");
   if (init_tasks( NUM_TASKS )) // number of tasks and size to allocate for all stacks
  {
       printf("init tasks failed!");
       return;
  }
   create_task( Task1, NULL, STK_SIZE, 0);
   puts("Task1 created\n");
   test_speed();
   i=0;
   clr_event(&gevent);
   create_task( EventTask, NULL, STK_SIZE, 0);
   puts("Event task created\n");
   while (1)
   {
   	sleep(200);
      putchar('0');
      if (++i>=40)
      {
            set_event(&gevent);
            i=0;
      }
   }
}

int Task1( int Param )
{
   printf( "Task1 Startup\n" );
   swtime();

   create_task( StackTask, NULL, STK_SIZE, 0);
   puts("Stack task created\n");

   while (1)
   {
   	sleep(1000l);
      putchar('1');
   }
   	return 0;
}

int StackTask(int param)
{
	int i,ssize;
   unsigned long stime;
   while(1)
   {
   	// verify sleep() is accurate
      stime=MS_TIMER;
      sleep(15000L);
      printf("\nStack check: xavail()=%ld, sleep(150000L) took %ld ms.\n",xavail(NULL),MS_TIMER-stime);
      for (i=0;i<task_count;i++)
      {
      	printf("Task %d has %d unused of %d.\n",i,check_stack(i),tasks[i].stack_size);
      }
   }
}

int EventTask( int Param )
{
   printf( "Event task Startup\n" );

   while (1)
   {
   	if (wait_event(&gevent,5000l))
      	puts("Event task woke up on event flag.\n");
      else
      	putchar('E');
      clr_event(&gevent);
   }
  	return 0;
}

